/// This a program to confirm that the B-tau nu code in the MCMC program is giving ChiSquared results in line with those of Ellis et al. in 0706.0652
///
///


#include <iostream>
#include "btnu.h"
#include "mycomplex.h"
#include "def.h" 
#include "linalg.h"
#include "lowe.h"
#include "rge.h"
#include "softsusy.h"
#include "softpars.h"
#include "susy.h"
#include "utils.h"
#include "numerics.h"

int MIXING = 0, PRINTOUT = 0;
double TOLERANCE = 1.0e-3;
double GMU = 1.6637e-5;
double QEWSB = 1.0;
bool INCLUDE_2_LOOP_SCALAR_CORRECTIONS = true; ///< DEBUG by Ben
int numHiggsMassLoops = 2, numRewsbLoops = 2;

int main()
{
  ///6 decimal place output.
  outputCharacteristics(6);
    cerr <<"Softsusy by Ben etc. \n";
  double m12 = 100., a0=-400., mGutGuess = 2.0e16, tanb = 50.0, m0 =300.;
  int sgnMu = 1;
  int numPoints = 100;  ///This is number of scan points.
  QedQcd oneset;
  

  double alphasMZ = 0.1187, mtop = 170.9, mbmb = 4.2;
  oneset.setAlpha(ALPHAS, alphasMZ);
  oneset.setPoleMt(mtop);
  oneset.setMass(mBottom, mbmb);
  oneset.toMz();
  
  int i;
  double startm12 = 120, endm12 = 500;
 for (i=0; i<=numPoints; i++)
   {
     m12 = (endm12 - startm12) / double(numPoints) * double(i) + startm12;
     MssmSoftsusy r;
     DoubleVector pars(3);
     pars(1) = 300; pars(2) = m12; pars(3) =2*m12 ;
     bool uni = true;

     r.lowOrg(sugraBcs, mGutGuess, pars, sgnMu, tanb, oneset, uni);
     double Btnu = BtnuCalc(r);
     /// if(!r.displayProblem().test())
	  cout << m12 << " " << Btnu<<"\n";
   }

}


/// This function is a part of the B->tau nu calculation.
double epsilonCalc(const MssmSoftsusy & m)
{
  double mgluino = m.displayPhys().mGluino;
  double msql    = m.displayPhys().md.display(1,1);
  double msqr    = m.displayPhys().md.display(2,1);
  double mu      = m.displaySusyMu();
  double alphas  = m.displayDataSet().displayAlpha(ALPHAS);
  double Htwo;
  double epsilonzero;

  double x = (msql * msql) / ( mgluino * mgluino ) ;
  double y = (msqr * msqr) / ( mgluino * mgluino) ;
 

  Htwo = ( x * log(x)) / ((1-x) * (x-y)) + (y * log(y)) / ((1-y) * (y-x)); 
  epsilonzero = ( -2 * alphas * mu * Htwo ) / ( 3 * PI * mgluino );
  cout <<"epsilon zero:" << epsilonzero <<"\n";
  return epsilonzero;
}

/// This function calculates the branching ratio B+-> tau+ nu.
double BtnuCalc(const MssmSoftsusy & m)
{
  double higgs   = m.displayPhys().mhiggs.display(4);
  double tanb    = m.displayTanb();
  double mB      = 5.279;                                        /// From PDG 2007, mass of B+ meson.
  double BRSusy, BRTotal;
  double epsilonzero;

  epsilonzero = epsilonCalc(m);

  /// The susy contribution, from hep-ph/0605012
  BRSusy = 1 - ( (mB * mB) / ( higgs * higgs )) * (( tanb * tanb) / ( 1 + epsilonzero * tanb));
  cout <<"mb/mh" << mB/higgs <<"\n";
  BRSusy = BRSusy * BRSusy;
 
  BRTotal = BRSusy;
  return BRTotal;
}
