/// This a program to confirm that the DeltaMbs code in the MCMC program is giving ChiSquared results in line with those of Ellis et al. in 0706.0652
///
///


#include <iostream>
#include "deltambs.h"
#include "mycomplex.h"
#include "def.h" 
#include "linalg.h"
#include "lowe.h"
#include "rge.h"
#include "softsusy.h"
#include "softpars.h"
#include "susy.h"
#include "utils.h"
#include "numerics.h"

int MIXING = 0, PRINTOUT = 0;
double TOLERANCE = 1.0e-3;
double GMU = 1.6637e-5;
double QEWSB = 1.0;
bool INCLUDE_2_LOOP_SCALAR_CORRECTIONS = true; ///< DEBUG by Ben
int numHiggsMassLoops = 2, numRewsbLoops = 2;
double DeltaMbsSig = 0.222;

int main()
{
  ///6 decimal place output.
  outputCharacteristics(6);
    cerr <<"Softsusy by Ben etc. \n";
  double mGutGuess = 2.0e16, tanb = 50.0, m0 =200.;
  double m12;
  int sgnMu = 1;
  int numPoints = 100;  ///This is number of scan points.
  QedQcd oneset;
  

  double alphasMZ = 0.1187, mtop = 171.4, mbmb = 4.25;
  oneset.setAlpha(ALPHAS, alphasMZ);
  oneset.setPoleMt(mtop);
  oneset.setMass(mBottom, mbmb);
  oneset.toMz();
  
  int i;
  double startm12 = 120, endm12 = 500;
 for (i=0; i<=numPoints; i++)
   {
     m12 = (endm12 - startm12) / double(numPoints) * double(i) + startm12;
     MssmSoftsusy r;
     DoubleVector pars(3);
     pars(1) = m0; pars(2) = m12; pars(3) = 0.;
     bool uni = true;

     r.lowOrg(sugraBcs, mGutGuess, pars, sgnMu, tanb, oneset, uni);
     // double sinSqThEff = r.sinSqThetaEff();
          double sinSqThEff = 0.23;
	   double DeltaMbs = DeltaMbsCalc(r, sinSqThEff);
	   /// double ma  = r.displayPhys().mhiggs.display(2);
	    cout << m12 << " " << DeltaMbs << "\n";

   }

}


/// This function is a part of the B->tau nu calculation.
double epsilonCalc(const MssmSoftsusy & m)
{
  double mgluino = m.displayPhys().mGluino;
  double msql    = m.displayPhys().md.display(1,1);
  double msqr    = m.displayPhys().md.display(2,1);
  double mu      = m.displaySusyMu();
  double alphas  = m.displayDataSet().displayAlpha(ALPHAS);
  double Htwo;
  double epsilonzero;

  double x = (msql * msql) / ( mgluino * mgluino ) ;
  double y = (msqr * msqr) / ( mgluino * mgluino) ;
 

  Htwo = ( x * log(x)) / ((1-x) * (x-y)) + (y * log(y)) / ((1-y) * (y-x)); 
  epsilonzero = ( -2 * alphas * mu * Htwo ) / ( 3 * PI * mgluino );
  return epsilonzero;
}

///This is the calculation of Delta_(Mbs), the B meson mass splitting.
/// This calculates epsilon_Y, from 0605012.
double epsilonYCalc(const MssmSoftsusy & m)
{
double mu       = m.displaySusyMu();
double msql     = m.displayPhys().md.display(1,1);
double msqr     = m.displayPhys().mu.display(2,1);
double Au       = m.displayTrilinear(UA,3,3);
double Htwo;
double epsilonY;

double x = (msql * msql) / ( mu * mu ) ;
double y = (msqr * msqr) / ( mu * mu ) ;
 
Htwo = ( x * log(x)) / ((1-x) * (x-y)) + (y * log(y)) / ((1-y) * (y-x)); 

epsilonY = (-1 * Au * Htwo) / (16 * PI * PI * mu );
///cout << "Epsilon Y:"<<epsilonY<<"\n";

return epsilonY;
}

double DeltaMbsCalc(const MssmSoftsusy & m, double sinSqTh)
{
 double lambdat      = m.displayYukawaElement(YU,3,3);
 double alpha       = m.displayDataSet().displayAlpha(ALPHA);
 double ma          = m.displayPhys().mhiggs.display(2);
 double epsilonzero = epsilonCalc(m);
 double epsilonY    = epsilonYCalc(m);
 double tanb        = m.displayTanb();
 double DeltaMbSusy = 0.0;
 double Total = 0.0;
 double mb = m.displayDataSet().displayMass(mBottom);
 ///cout << "mb:"<< mb << "\n";

 ///Wilson coefficient calculation, from hep-ph/9512380;
 double S0;
 double mt = m.displayDataSet().displayMass(mTop);
 double mw = m.displayMw();
 double xt = (mt * mt)/(mw * mw);
 S0 = (4 * xt - 11*xt*xt + xt*xt*xt)/(4*(1-xt)*(1-xt)) - ( 3*xt*xt*xt*log(xt))/(2*(1-xt)*(1-xt)*(1-xt));
 

 ///Calculate ms(mb)
 QedQcd oneset;
 oneset.setMass(mStrange, mb);
 oneset.setMass(mBottom,mb);
 double msmb = m.displayDataSet().displayMass(mStrange);
 double mbmb = m.displayDataSet().displayMass(mBottom);

 
 DeltaMbSusy = ( epsilonY * lambdat * lambdat * tanb *tanb )/((1+(epsilonzero + (epsilonY * lambdat * lambdat))*tanb) * ( 1 + epsilonzero * tanb ));

 DeltaMbSusy = DeltaMbSusy * DeltaMbSusy;
 
 DeltaMbSusy = ( mbmb * msmb * 64 * PI  * sinSqTh * DeltaMbSusy) / ( alpha * ma * ma * S0 );
 
 DeltaMbSusy = 1 - DeltaMbSusy;
 
 ///cout << "ms(mb):" << msmb << " " <<"mbmb:" << mbmb << " " << "ma:"<< ma <<"\n"; 
 Total = 1- (mbmb * msmb)/( ma *ma);
 /// cout << m12 << " " << ma << "\n";
 

 ///double R = 1.2046;
 ///Total = DeltaMbSusy;
 return Total;
}
